﻿using System;
using System.Collections.Generic;
using System.Windows.Forms;
using RootPro.RootProCAD;
using RootPro.RootProCAD.Command;
using System.Threading;
using System.Reflection;
using System.IO;

namespace RCAddInFlattenLayersCS
{
    [System.AddIn.AddIn("AppAddIn", Version = "1.0", Publisher = "", Description = "")]
    public partial class AppAddIn
    {
        private void AppAddIn_Startup(object sender, EventArgs e)
        {
            AppDomain.CurrentDomain.AssemblyResolve += new ResolveEventHandler(ResolveAssembly);

            CommandManager.AddMacroCommand("RCAddInFlattenLayers", Properties.Resources.CommandName, MacroCommandFlattenLayers);
        }

        private void AppAddIn_Shutdown(object sender, EventArgs e)
        {
            CommandManager.RemoveMacroCommand(MacroCommandFlattenLayers);
        }

        #region VSTA generated code
        private void InternalStartup()
        {
            this.Startup += new System.EventHandler(AppAddIn_Startup);
            this.Shutdown += new System.EventHandler(AppAddIn_Shutdown);
        }
        #endregion

        private Assembly ResolveAssembly(object sender, ResolveEventArgs args)
        {
            AssemblyName assemblyName = new AssemblyName(args.Name);

            foreach(Assembly assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                if(AssemblyName.ReferenceMatchesDefinition(assemblyName, assembly.GetName()))
                    return assembly;
            }

            if(assemblyName.Name.EndsWith(".resources"))
            {
                if(assemblyName.CultureName.Equals(Thread.CurrentThread.CurrentUICulture.Name, StringComparison.OrdinalIgnoreCase))
                {
                    string dllPath = Path.GetDirectoryName(this.AddInFullPath);
                    dllPath = Path.Combine(dllPath, assemblyName.CultureName);
                    dllPath = Path.Combine(dllPath, assemblyName.Name + ".dll");

                    if(File.Exists(dllPath))
                    {
                        try
                        {
                            return Assembly.LoadFrom(dllPath);
                        }
                        catch
                        {
                        }
                    }
                }
            }

            return null;
        }

        /// <summary>
        /// コマンド処理
        /// </summary>
        private void MacroCommandFlattenLayers()
        {
            // 現在のドキュメントを取得
            var doc = ActiveDocument;
            // レイヤテーブルを取得
            var layerTalbe = doc.LayerTable;
            // ルートレイヤを取得
            var rootLayer = layerTalbe.RootLayer;
            // ルートレイヤのコレクションを取得
            var rootLayerCollection = rootLayer.ChildLayers;

            // グループレイヤが存在するかチェックする
            bool isExistGroup = false;
            foreach(var layer in rootLayerCollection)
            {
                if(layer.IsGroup)
                {
                    isExistGroup = true;
                    break;
                }
            }

            // グループがない場合はフラットなのでメッセージを表示する
            if(!isExistGroup)
            {
                MessageBox.Show(Properties.Resources.NotExistGroupWarning);
                return;
            }

            // レイヤ名の重複チェック
            HashSet<string> hsTable = new HashSet<string>();
            string duplicateName;
            if((duplicateName = IsLayerNameDuplicate(hsTable, rootLayerCollection)) != string.Empty)
            {
                MessageBox.Show(String.Format(Properties.Resources.NameDuplicationWarning, duplicateName));
                return;
            }

            if(MessageBox.Show(
                Properties.Resources.ConfirmMessage,
                "",
                MessageBoxButtons.OKCancel) == DialogResult.Cancel)
                return;

            // Undo可能操作開始
            doc.UndoManager.BeginUndoUnit();

            bool done;
            do
            {
                done = false;

                for(int i = 0; i < rootLayerCollection.Count; i++)
                {
                    // グループレイヤの場合
                    if(rootLayerCollection[i].IsGroup)
                    {
                        // 子レイヤがない場合はグループを削除
                        if(GetChildLayersCount(rootLayerCollection[i].ChildLayers) == 0)
                        {
                            rootLayerCollection.RemoveAt(i);
                            done = true;
                            break; ;
                        }
                        // 子レイヤをルートレイヤの現在の位置に持ってくる
                        done = MoveChildLayerToRoot(i, rootLayerCollection[i].ChildLayers);
                        break;
                    }
                }

            } while(done);

            // Undo可能操作終了
            doc.UndoManager.EndUndoUnit();

        }


        /// <summary>
        /// レイヤ名の重複をチェックする
        /// </summary>
        private string IsLayerNameDuplicate(HashSet<string> hsTable, LayerCollection layers)
        {
            string dupLayerName = string.Empty;
            foreach(var layer in layers)
            {
                // グループレイヤの場合は子レイヤのチェックを行う
                if(layer.IsGroup)
                {
                    dupLayerName = IsLayerNameDuplicate(hsTable, layer.ChildLayers);
                }
                else
                {
                    // 同じ名前のレイヤが存在する場合
                    if(!hsTable.Add(layer.Name))
                        dupLayerName = layer.Name;
                }

                if(dupLayerName != string.Empty)
                    return dupLayerName;
            }

            return string.Empty;
        }

        /// <summary>
        /// 子レイヤの個数を取得する
        /// </summary>
        private int GetChildLayersCount(LayerCollection layers)
        {
            int num = 0;
            foreach(var layer in layers)
            {
                if(layer.IsGroup)
                    num += GetChildLayersCount(layer.ChildLayers);
                else
                    num++;
            }
            return num;
        }

        /// <summary>
        /// 子レイヤをひとつルートレイヤに移動する
        /// </summary>
        private bool MoveChildLayerToRoot(int index, LayerCollection layers)
        {
            bool moved = false;
            var rootLayer = ActiveDocument.LayerTable.RootLayer;

            if(index >= rootLayer.ChildLayers.Count)
                return false;

            foreach(var layer in layers)
            {
                if(layer.IsGroup)
                {
                    if(GetChildLayersCount(layer.ChildLayers) == 0)
                        continue;
                    
                    // 子レイヤに同じ処理を行う
                    moved = MoveChildLayerToRoot(index, layer.ChildLayers);
                    return moved;
                }
                else
                {
                    // レイヤをルートレイヤに移動する
                    layer.MoveLayer(rootLayer, index);
                    return true;
                }
            }
            return moved;
        }

    }
}
